from warnings import warn
from glob import glob
from os import path
import matplotlib.pyplot as plt
import matplotlib.colors as mc
import numpy as np
from PseudoNetCDF import pncopen

# START CONFIGURATION
#cloudplot = False # make fine aerosol plot
cloudplot = True

# To just make netCDF outputs, set plotit = False
plotit = True

# To use CMAQ native map, set usecmaqmap = True
usecmaqmap = False

# To change paths used for calculation, update the glob path below
if cloudplot:
     # Cloud settings
     inpaths = glob('/work/MOD3DEV/kfa/KMTPLUS/POST/EXTRACT_TIMESERIES/PH-STUFF/PH_2016-vert-NEW-NEWNH3.ncf')
     # Color bar settings
     mincolor = 1.5
     maxcolor = 8.0 #
     inccolor = 0.5
     colorbaredges = np.arange(mincolor, maxcolor, inccolor)
     colorbarticks = np.arange(mincolor+0.5,maxcolor-0.5,1)
     norm = mc.BoundaryNorm(colorbaredges, 256)
     # Using Mary Barth's colors
     cmap = plt.cm.jet
     Ncol=12
     customcolors =[
     (150/256,   0/256,   0/256, 1.0),
     (234/256,   0/256,   0/256, 1.0),
     (255/256,  62/256,   0/256, 1.0),
     (255/256, 112/256,   0/256, 1.0),
     (255/256, 155/256,   0/256, 1.0),
     (255/256, 198/256,  23/256, 1.0),
     (184/256, 225/256, 245/256, 1.0),
     (101/256, 198/256, 243/256, 1.0),
     (  7/256, 178/256, 255/256, 1.0),
     ( 37/256, 134/256, 249/256, 1.0),
     ( 56/256,  93/256, 229/256, 1.0),
     ( 14/256,   3/256, 253/256, 1.0),
     ]
     titletxt='CMAQ Res Cloud $\overline{pH_F}$'

else:
     # Fine aerosol settings
     inpaths = glob('/work/MOD3DEV/has/postproc/2016/output/CCTM_v52_hdifupdate_intel17.0_HEMIS_cb6_combine_inorg.annual.2016')
     # Color bar settings
     mincolor = -0.5
     maxcolor = 8.5 #
     inccolor = 0.5
     colorbaredges = np.arange(mincolor, maxcolor, inccolor)
     colorbarticks = np.arange(mincolor+0.5,maxcolor-0.5,1)
     norm = mc.BoundaryNorm(colorbaredges, 256)
     # Using Mary Barth's colors
     cmap = plt.cm.jet
     Ncol=17
     customcolors =[
     (150/256,   0/256,   0/256, 1.0),
     (205/256,  13/256,  13/256, 1.0),
     (234/256,   0/256,   0/256, 1.0),
     (255/256,   0/256,   0/256, 1.0),
     (255/256,  62/256,   0/256, 1.0),
     (255/256, 112/256,   0/256, 1.0),
     (255/256, 155/256,   0/256, 1.0),
     (255/256, 198/256,  23/256, 1.0),
     (255/256, 231/256,  71/256, 1.0),
     (203/256, 234/256, 248/256, 1.0),
     (164/256, 217/256, 242/256, 1.0),
     (101/256, 198/256, 243/256, 1.0),
     (  7/256, 178/256, 255/256, 1.0),
     ( 37/256, 134/256, 249/256, 1.0),
     ( 56/256,  93/256, 229/256, 1.0),
     ( 14/256,   3/256, 253/256, 1.0),
     ( 86/256,  20/256, 245/256, 1.0)
     ]
     titletxt='CMAQ Fine Aerosol $\overline{pH_F}$'


cmap=cmap.from_list('Barth cmap', customcolors, cmap.N)

# END CONFIGURATION

# START PROCESSING
for inpath in inpaths:
    figpath = path.basename(inpath) + '_cmaq_pH.png'
    outpath = path.basename(inpath) + '_cmaq_pH.nc'
    f = ifile = pncopen(inpath, format='ioapi')
    print('Opened', flush=True)

    if cloudplot:
         phf4d = f.subsetVariables('NEWPHavgv'.split())
         print('Extracted', flush=True)
         phf = phf4d.sliceDimensions(LAY=0)
         warn('masking values where cloud pH is below -1')
         phf.eval("""
pHIJgood = np.ma.masked_where( NEWPHavgv <= -1.0, NEWPHavgv )
pHIJgood.units = 'pH'
""", inplace=True)
         print('Evaluated', flush=True)
    else: 
         phf4d = f.subsetVariables('AH2OIJ pHIJ'.split())
         print('Extracted', flush=True)
         phf = phf4d.sliceDimensions(LAY=0)
         print('Sliced', flush=True)
         warn('masking values where aerosol water content is below 0.01 ug/m3')
         phf.eval("""
watermask = np.ma.masked_less( AH2OIJ, 0.01 )
watermask.units = 'ug/m3'
pHIJgood = np.ma.masked_where( AH2OIJ < 0.01, pHIJ)
pHIJgood.units = 'pH'
""", inplace=True)
         print('Evaluated', flush=True)
         phf = phf.applyAlongDimensions(TSTEP='mean')
         print('Meaned', flush=True)

    phf.save(outpath)
    print('Saved', outpath, flush=True)
    if not plotit:
        continue

    plt.close()
    x = np.arange(phf.NCOLS + 1) * phf.XCELL
    y = np.arange(phf.NROWS + 1) * phf.YCELL
    lon, lat = phf.xy2ll(*np.meshgrid(x, y))
    # lon = lon % 360
    C = phf.variables['pHIJgood'][:].squeeze().array()
    if usecmaqmap:
        # Add maps - use standard HCMAQ projection
        fig = plt.figure(figsize=(8, 6))
        bmap = phf.getMap()
        X = x
        Y = y
        corientation = 'vertical'
    else:
        from mpl_toolkits.basemap import Basemap
        fig = plt.figure(figsize=(12, 6))
        bmap = Basemap(
            projection='cyl', llcrnrlat=-16, llcrnrlon=-180.0,
            urcrnrlat=90, urcrnrlon=180
        )
        # Convert lon to X,Y
        X, Y = bmap(lon, lat, inverse=True)

        # Find grid cells that cross map boundary
        # Using a coarse algorithm: do any vertices change
        # dramatically in longitude
        maskme_x01 = np.abs(np.diff(X[:-1], axis=1)) > 90
        maskme_x23 = np.abs(np.diff(X[1:], axis=1)) > 90
        maskme_x12 = np.abs(np.diff(X[:, 1:], axis=0)) > 90
        maskme_x03 = np.abs(np.diff(X[:, :-1], axis=0)) > 90
        maskme = maskme_x01 | maskme_x12 | maskme_x23 | maskme_x03
        C = np.ma.masked_where(maskme, C)
        corientation = 'horizontal'

    p = bmap.pcolormesh(X, Y, C, norm=norm, cmap=cmap)
    ax = p.axes
    if len(fig.axes) != 2:
        cax = None
    else:
        cax = fig.axes[1]
    fig.colorbar(
        p, label='$pH_F$', ticks=colorbarticks,
        cax=cax, orientation=corientation
    )
    ax.set_title(titletxt)
    bmap.drawcoastlines(ax=ax)
    print('Plotted', flush=True)
    fig.text(0, 0, path.basename(inpath))
    fig.savefig(figpath)
    print('Saved Fig', figpath, flush=True)

# END PROCESSING
